package org.apidesign.apifest08.currency;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Currency;
import static org.apidesign.apifest08.currency.Assert.*;


/**
 * A bid representation. A did is defined as two currenncies and its value. 
 */
public final class Bid {
	private final Currency first;
	private final Currency second;
	private final BigDecimal bidValue; // a bid between the first currency and the second currency
	public static final BigDecimal one = new BigDecimal(1);		
	
	Bid(Currency first, Currency second, BigDecimal bid) {
		notNull(first, "first");
		notNull(second, "second");
		this.first = first;
		this.bidValue = bid;
		this.second = second;	
	}
	
	Bid(Currency first, Currency second) {
		notNull(first, "first");
		notNull(second, "second");
		this.first = first;
		this.second = second;
		this.bidValue = null;
	}
	
			
	BigDecimal getBidValue(Currency from, Currency to) {
		if((from != first || from != second) || (to != first || to != second)) {
			new IllegalArgumentException("This bid can be used only for: " + first + " " + second);
		}
		
		notNull(bidValue, "bidValue");
		
		BigDecimal retVal;
		
		if(first == from) {
			retVal = bidValue;
		} else {	
			//reverse bid		
			retVal = one.divide(bidValue, 10 ,RoundingMode.HALF_UP);
		}
		
		return retVal;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + first.hashCode() +  second.hashCode();
		return result;
	}
	
	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Bid other = (Bid) obj;
		
		return (this.first == other.first || this.first == other.second) && (this.second == other.second || this.second == other.first) ;
	}		
}